@@ -21,7 +21,7 @@ elif DEVICE == "cuda":
                 check=True
             )
     # Set accelerators
-    os.environ["CUPY_ACCELERATORS"] = "cub,cutensor"  # or "cub,cutensor" if you want both
+    os.environ["CUPY_ACCELERATORS"] = "cub,cutensor,cutensornet"  # or "cub,cutensor" if you want both
     import cupy as xp
     # xp.cuda.set_allocator(None)  # Use default memory pool
     # xp.cuda.set_pinned_memory_allocator(None)  # Use default pinned memory pool
@@ -44,6 +44,7 @@ from .functions.reductions import sum, mean, max, min, std, var
 from .functions.conv import conv2d, pool2d, maxpool2d, averagepool2d, pooling2d, maxpooling2d, averagepooling2d
 from .tensor import Tensor, ones, zeros, ones_like, zeros_like, empty, arange, eye
 from .amp import autocast, GradScaler
+from .utils.no_grad import no_grad, is_grad_enabled, set_grad_enabled
 from .utils.graph import visualize_graph, save_graph, print_graph_structure
 import gc
 
