@@ -1,11 +1,12 @@
 from neurograd import xp
-from neurograd import xp
+from base import Function
 from typing import TYPE_CHECKING, Union, Tuple, Sequence, Literal
 from numpy.typing import ArrayLike
 if TYPE_CHECKING:
     from neurograd.tensor import Tensor
-if TYPE_CHECKING:
-    from neurograd.tensor import Tensor
+from neurograd.functions.tensor_ops import Pad, SlidingWindowView, Reshape
+from neurograd.functions.linalg import MatMul, Transpose
+
 
 
 
@@ -30,7 +31,6 @@ def conv2d(input: Union["Tensor", xp.ndarray], filters: Union["Tensor", xp.ndarr
         F_N, F_H, F_W = filters.shape
         assert F_N == C, "For depthwise convolution, number of filters must match number of input channels."
 
-
     if isinstance(strides, int):
         strides = (strides, strides)
     
@@ -49,6 +49,7 @@ def conv2d(input: Union["Tensor", xp.ndarray], filters: Union["Tensor", xp.ndarr
 
     out_H = (H + pad_h - F_H) // sh + 1
     out_W = (W + pad_w - F_W) // sw + 1
+    
     if pad_h > 0 or pad_w > 0:
         ph1, pw1 = pad_h // 2, pad_w // 2
         input = ng.pad(
@@ -58,20 +59,52 @@ def conv2d(input: Union["Tensor", xp.ndarray], filters: Union["Tensor", xp.ndarr
             memsave=True
         )
         
-    # Create a fresh sliding window view op per call
+    # Create sliding window view
     slider = SlidingWindowView(window_shape=(F_H, F_W), strides=strides, axes=(2, 3))
+    slides = slider(input)  # (N, C, out_H, out_W, F_H, F_W)
+    
     if not depthwise:
-        slides = slider(input)  # (N, C, out_H, out_W, F_H, F_W)
-        filters = filters # (F_N, C, F_H, F_W)
-        output = ng.einsum("ncpqhw,fchw->nfpq", slides, filters, backend="opt_einsum") # (N, F_N, out_H, out_W) 
+        # Standard convolution using GEMM
+        # Reshape slides: (N, C, out_H, out_W, F_H, F_W) -> (N * out_H * out_W, C * F_H * F_W)
+        slides_2d = ng.reshape(
+            ng.transpose(slides, axes=(0, 2, 3, 1, 4, 5)),  # (N, out_H, out_W, C, F_H, F_W)
+            (N * out_H * out_W, C * F_H * F_W)
+        )
+        
+        # Reshape filters: (F_N, C, F_H, F_W) -> (C * F_H * F_W, F_N)
+        filters_2d = ng.transpose(
+            ng.reshape(filters, (F_N, C * F_H * F_W)),
+            axes=(1, 0)
+        )
+        
+        # GEMM: (N * out_H * out_W, C * F_H * F_W) @ (C * F_H * F_W, F_N) -> (N * out_H * out_W, F_N)
+        output = ng.matmul(slides_2d, filters_2d)
+        
+        # Reshape back: (N * out_H * out_W, F_N) -> (N, F_N, out_H, out_W)
+        output = ng.transpose(
+            ng.reshape(output, (N, out_H, out_W, F_N)),
+            axes=(0, 3, 1, 2)
+        )
     else:
-        slides = slider(input)  # (N, C, out_H, out_W, F_H, F_W)
-        filters = filters # (C, F_H, F_W)
-        output = ng.einsum('ncpqhw,chw->ncpq', slides, filters, backend="opt_einsum") # (N, C, out_H, out_W)
+        # Depthwise convolution - each channel gets its own filter
+        # slides: (N, C, out_H, out_W, F_H, F_W)
+        # filters: (C, F_H, F_W)
+        # Reshape slides -> (N*out_H*out_W, C, F_H*F_W)
+        slides_reshaped = ng.reshape(
+            slides,
+            (N * out_H * out_W, C, F_H * F_W)
+        )
+        # Reshape filters -> (C, F_H*F_W, 1)
+        filters_reshaped = ng.reshape(filters, (C, F_H * F_W, 1))
+        # Batched matmul: (N*out_H*out_W, C, F_H*F_W) @ (C, F_H*F_W, 1)
+        # Result: (N*out_H*out_W, C, 1)
+        output = ng.matmul(slides_reshaped, filters_reshaped)
+        # Reshape back -> (N, C, out_H, out_W)
+        output = ng.reshape(output, (N, out_H, out_W, C))
+        output = ng.transpose(output, (0, 3, 1, 2))  # (N, C, out_H, out_W)
     return output
 
 
-
 def pool2d(input: Union["Tensor", xp.ndarray], 
            pool_size: Union[int, Tuple[int, ...]],
            strides: Union[int, Tuple[int, ...]] = (1, 1),
@@ -123,10 +156,10 @@ def pool2d(input: Union["Tensor", xp.ndarray],
     input = ng.pad(input, pad_width=padding, mode='constant', constant_values=padding_value,
                    memsave=True)
 
-    # Create a fresh sliding window view op per call
+    # Create sliding window view
     slider = SlidingWindowView(window_shape=(P_H, P_W), strides=strides, axes=(2, 3))
     slides = slider(input)  # (N, C, out_H, out_W, P_H, P_W)
-    output = pooling_fn(slides, axis=(4, 5), keepdims=False) # output shape: (N, C, out_H, out_W) # (4, 5) OR (-2, -1)
+    output = pooling_fn(slides, axis=(4, 5), keepdims=False) # (N, C, out_H, out_W)
     
     return output
 
@@ -142,7 +175,7 @@ def maxpool2d(input: Union["Tensor", xp.ndarray],
 
 def averagepool2d(input: Union["Tensor", xp.ndarray], 
                   pool_size: Union[int, Tuple[int, ...]],
-                  strides: Union[int, Tuple[int, ...]] = (2, 2),
+                  strides: Union[int, Tuple[int, ...]] = (1, 1),  # Fixed default stride
                   padding: Union[Sequence, ArrayLike, int, Literal["valid", "same"]] = (0, 0),
                   padding_value: Union[int, float] = 0):
     import neurograd as ng
@@ -152,6 +185,4 @@ def averagepool2d(input: Union["Tensor", xp.ndarray],
 # Set aliases
 pooling2d = pool2d
 maxpooling2d = maxpool2d
-averagepooling2d = averagepool2d
-
-
+averagepooling2d = averagepool2d
\ No newline at end of file
